(* 
 * Type Declaraction for
 *   Abstract Syntax Tree 
 * Author: Lixing Dong
 * Email: cee.xingweimian@gmail.com
 *)


(*
 * Short:
 * 
 * var  - variable
 * def  - definition
 * dec  - declaration
 * obj  - object
 * cst  - const
 * stc  - sentence
 * func - function
 * expr - expression
 * stmt - statement
 * 
 *)

let lineno = ref 1;;



(*rules*)
type program = stc list


and stc =
    StcOfObjDef of obj_def
    | StcOfFuncDef of func_def
    | StcOfStmt of stmt


and func_def =
    FuncDefOfFuncDecAndStcList of func_dec * cltype * in_func_stc list
    

and func_dec =
    FuncDeclaratorOfIDAndParameters of string * parameter list


and parameter =
    ParameterOfTypeAndID of cltype * string


and in_func_stc = 
    InFuncStcOfStmt of stmt


and obj_def = 
    ObjDefOfIDAndStcList of string * in_obj_stc list


and in_obj_stc =
    InObjStcOfVarDec of var_dec


and cltype = 
    CLTypeOfInt
    | CLTypeOfReal
    | CLTypeOfBoolean
    | CLTypeOfObjID of string
    | CLTypeOfVoid
    | CLTypeOfArray of cltype * int
    | CLTypeOfUndeclare


and var_dec = 
    VarDecOfTypeAndDeclaratorList of cltype * declarator list


and declarator = 
    DeclaratorOfID of string * int 
    | DeclaratorOfArray of string * int
    | DeclaratorOfFixedArray of string * cst_expr * int


and stmt = 
    StmtOfVarDec of var_dec
    | StmtOfExprStmt of expr_stmt
    | StmtOfSelectionStmt of selection_stmt
    | StmtOfIterationStmt of iteration_stmt
    | StmtOfJumpStmt of jump_stmt
    | StmtOfDrawStmt of draw_stmt


and expr_stmt =
    ExprStmtOfExpr of expr


and draw_stmt = 
    DrawStmtOfFD of expr
    | DrawStmtOfBK of expr
    | DrawStmtOfLF of expr
    | DrawStmtOfRT of expr
    | DrawStmtOfReset
    | DrawStmtOfCls
    | DrawStmtOfPf
    | DrawStmtOfPd
    | DrawStmtOfPu
    | DrawStmtOfWd of expr
    | DrawStmtOfRGB of expr * expr * expr


and selection_stmt = 
    SelectionStmtOfExprAndStmtList of expr * stmt list
    | SelectionStmtOfExprAndStmtList2 of expr * stmt list * stmt list


and iteration_stmt =
    IterationStmtOfExprAndStmtList of expr * stmt list
    | IterationStmtOfStmtList of stmt list


and jump_stmt =
    JumpStmtOfGOON of int
    | JumpStmtOfBRK of int
    | JumpStmtOfReturnExpr of expr
    | JumpStmtOfReturn of int


and expr =
    ExprOfPrimary of primary * int
    | ExprOfCstExpr of cst_expr * int
    | ExprOfAssign of lvalue * expr 
    | ExprOfPlus of expr * expr
    | ExprOfMinus of expr * expr 
    | ExprOfTimes of expr * expr 
    | ExprOfDivide of expr * expr 
    | ExprOfNeq of expr * expr 
    | ExprOfEq of expr * expr 
    | ExprOfLt of expr * expr 
    | ExprOfGt of expr * expr 
    | ExprOfLeq of expr * expr 
    | ExprOfGeq of expr * expr 
    | ExprOfUMinus of expr 
    | ExprOfNot of expr
    | ExprOfFunction of string * expr list * int


and identifier =
    | IdentifierOfSimple of string * int
    | IdentifierOfObjElement of string * identifier * int


and primary = 
    | PrimaryOfIdentifier of identifier * int
    | PrimaryOfArrayElement of identifier * expr


and cst_expr = 
    CstExprOfInt of int
    | CstExprOfReal of float
    | CstExprOfTrue
    | CstExprOfFalse


and lvalue = 
    LvalueOfIdentifier of identifier * int
    | LvalueOfArrayElement of identifier * expr





