%{
open Header
open Ast

let parsing_succeed = ref true



let nullexpr = ExprOfCstExpr(CstExprOfInt(0), 0)


let nullstmtlist = [StmtOfExprStmt(ExprStmtOfExpr(nullexpr))]

%}

%token SEMI COMMA PLUS MINUS TIMES DIVIDE COLON
%token RETURN IF ELSE FOR WHILE INT EOF REAL END
%token LBRACK RBRACK LPAREN RPAREN LBRACE RBRACE

/* Drawing Related Statements */
%token FD BK RT LF RESET CLS BK RAND PU PD PF RGB WD

%token OBJ BOOL BRK CONDITION FX GOON

%token ASSIGN

%token DOT

/*boolean operator*/
%token EQ NEQ LT GT GEQ LEQ NOT

%token LOOP BREAK ENDC FUNC TRUE FALSE


%token <float> REALV
%token <int> INTV
%token <string> ID


%left COMMA

%right ASSIGN

/*binary operator*/
%nonassoc NOT
%left OR
%left AND
%left EQ NEQ
%left LT GT LEQ GEQ 
%left PLUS MINUS
%left TIMES DIVIDE MOD
%nonassoc UMINUS

/*primary_expression operators*/
%nonassoc ARRAY
%left DOT


%start program
%type <Ast.program> program
%%


program:
{ [] }
| free_sentence program { $1 :: $2 }
;


free_sentence:
| function_definition { StcOfFuncDef($1) }
| obj_definition { StcOfObjDef($1) }
| statement { StcOfStmt($1) }
;

/* function definition */

function_definition:
| FUNC function_declarator COLON cltype LBRACE in_func_sentence_list RBRACE
    { FuncDefOfFuncDecAndStcList($2, $4, $6) }
| FUNC function_declarator LBRACE in_func_sentence_list RBRACE
    { FuncDefOfFuncDecAndStcList($2, CLTypeOfVoid, $4) }
;


function_declarator:
| ID LPAREN parameter_list RPAREN { FuncDeclaratorOfIDAndParameters($1, $3) }
| ID LPAREN RPAREN { FuncDeclaratorOfIDAndParameters($1, []) }
;


parameter_list:
| parameter { [$1] }
| parameter COMMA parameter_list { $1 :: $3 }
;


parameter:
| cltype ID { ParameterOfTypeAndID($1, $2) }
;


in_func_sentence_list:
| in_func_sentence { [$1] }
| in_func_sentence in_func_sentence_list { $1 :: $2 }
;


in_func_sentence:
| statement { InFuncStcOfStmt($1) }
;


/* object definition*/

obj_definition:
| OBJ ID LBRACE in_obj_sentence_list RBRACE { ObjDefOfIDAndStcList($2, $4) }
;


in_obj_sentence_list:
| in_obj_sentence { [$1] }
| in_obj_sentence in_obj_sentence_list { $1::$2 }


in_obj_sentence:
| variable_declaration { InObjStcOfVarDec($1) }
;



/* declaractions */

variable_declaration:
| cltype declarator_list SEMI 
    { VarDecOfTypeAndDeclaratorList($1, $2) }
;


declarator_list:
| declarator { [$1] }
| declarator COMMA declarator_list { $1::$3 }
;

            
cltype:
| INT { CLTypeOfInt }
| REAL { CLTypeOfReal }
| ID { CLTypeOfObjID($1) }
| BOOL {CLTypeOfBoolean}
;

                        
declarator:
| ID { DeclaratorOfID($1, !lineno) }
| ID LBRACK RBRACK { DeclaratorOfArray($1, !lineno) }
| ID LBRACK constant_expression RBRACK { DeclaratorOfFixedArray($1, $3, !lineno) }
;
        
                        
statement:
| variable_declaration { StmtOfVarDec($1) }
| expression_statement { StmtOfExprStmt($1) }
| draw_statement { StmtOfDrawStmt($1) }
| iteration_statement  { StmtOfIterationStmt($1) }
| selection_statement  { StmtOfSelectionStmt($1) }
| jump_statement { StmtOfJumpStmt($1) }
;


expression_statement:
| expression SEMI { ExprStmtOfExpr($1) }
| expression error 
{ 
	Printf.printf "%s" "Syntax Error: Are you missing a ';'\n";
	parsing_succeed := false;
	ExprStmtOfExpr($1)
}


draw_statement:
| FD expression SEMI { DrawStmtOfFD($2) }
| BK expression SEMI { DrawStmtOfBK($2) }
| LF expression SEMI{ DrawStmtOfLF($2) }
| RT expression SEMI{ DrawStmtOfRT($2) }
| RESET SEMI{ DrawStmtOfReset }
| CLS SEMI{ DrawStmtOfCls }
| PF SEMI{ DrawStmtOfPf }
| PD SEMI{ DrawStmtOfPd }
| PU SEMI{ DrawStmtOfPu }
| WD expression SEMI { DrawStmtOfWd($2) }
| RGB expression COMMA expression COMMA expression SEMI
    { DrawStmtOfRGB($2, $4, $6) }
| RGB error SEMI
{
	Printf.printf "%s" "Syntax Error: RGB statement error\n";
	parsing_succeed := false;
	DrawStmtOfRGB(nullexpr, nullexpr, nullexpr)
}
;

    
selection_statement:
| IF LPAREN expression RPAREN statement_list END
    { SelectionStmtOfExprAndStmtList($3, $5) }
| IF LPAREN expression RPAREN statement_list ELSE statement_list END
    { SelectionStmtOfExprAndStmtList2($3, $5, $7) }
| IF LPAREN expression RPAREN error
    { 
        Printf.printf "%s\n" "Syntax Error: Are you missing an 'End'";
        parsing_succeed := false;
        SelectionStmtOfExprAndStmtList($3, [StmtOfExprStmt(ExprStmtOfExpr(ExprOfCstExpr(CstExprOfInt(0), 0)))] )
    }
| IF LPAREN expression RPAREN statement_list ELSE error
{ 
    Printf.printf "%s\n" "Syntax Error: Are you missing an 'End'";
    parsing_succeed := false;
    SelectionStmtOfExprAndStmtList($3, nullstmtlist )
}
;


statement_list:
| statement { [$1] }
| statement statement_list { $1::$2 }
;


iteration_statement:
| LOOP LPAREN expression RPAREN LBRACE statement_list RBRACE 
    { IterationStmtOfExprAndStmtList($3, $6) }
| LOOP LPAREN RPAREN LBRACE statement_list RBRACE 
    { IterationStmtOfStmtList($5) }
| LOOP LPAREN RPAREN LBRACE error
    { 
        Printf.printf "%s\n" "Syntax Error: '{' mismatch ?";
        parsing_succeed := false;
        IterationStmtOfStmtList(nullstmtlist)
    }
;


jump_statement:
| GOON SEMI { JumpStmtOfGOON(!lineno) }
| BREAK SEMI { JumpStmtOfBRK(!lineno) }
| RETURN expression SEMI { JumpStmtOfReturnExpr($2) }
| RETURN SEMI { JumpStmtOfReturn(!lineno) }
;

                        
expression:
| constant_expression { ExprOfCstExpr($1, !lineno) }
| LPAREN expression RPAREN { $2 }
| LPAREN error 
{
    Printf.printf "%s\n" "'(' mismatch";
    nullexpr 
}
| primary { ExprOfPrimary($1, !lineno) }
| lvalue ASSIGN expression { ExprOfAssign($1, $3) }
| expression PLUS expression{ ExprOfPlus($1, $3) }
| expression MINUS expression{ ExprOfMinus($1, $3) }
| expression TIMES expression{ ExprOfTimes($1, $3) }
| expression DIVIDE expression{ ExprOfDivide($1, $3) }
| expression NEQ expression { ExprOfNeq($1, $3) }
| expression EQ expression { ExprOfEq($1, $3) }
| expression LT expression { ExprOfLt($1, $3) }
| expression GT expression { ExprOfGt($1, $3) }
| expression LEQ expression { ExprOfLeq($1, $3) }
| expression GEQ expression { ExprOfGeq($1, $3) }
| MINUS expression %prec UMINUS { ExprOfUMinus($2) }
| NOT expression { ExprOfNot($2) }
| ID LPAREN argument_expression_list RPAREN { ExprOfFunction($1, $3, !lineno) }
;


identifier:
 | ID { IdentifierOfSimple($1, !lineno) }
 | ID DOT identifier { IdentifierOfObjElement($1, $3, !lineno)}
 ;


 primary:
 | identifier { PrimaryOfIdentifier($1, !lineno) }
 | identifier LBRACK expression RBRACK { PrimaryOfArrayElement($1, $3) }
 | identifier LBRACK error { 
     Printf.printf "%s\n" "'[' mismatch";
     PrimaryOfArrayElement($1, nullexpr) 
 }
 ;


lvalue:
| identifier { LvalueOfIdentifier($1, !lineno) }
| identifier LBRACK expression RBRACK { LvalueOfArrayElement($1, $3) }
| identifier LBRACK error 
{
    Printf.printf "%s\n" "'[' mismatch";
    LvalueOfArrayElement($1, nullexpr)
}
;


argument_expression_list:
| expression { [$1] }
| expression COMMA argument_expression_list { $1 :: $3 }
;


constant_expression:
| INTV  { CstExprOfInt($1) }
| REALV { CstExprOfReal($1) }
| TRUE  { CstExprOfTrue }
| FALSE { CstExprOfFalse }
;






